function [lsbmag,value]=lsb(x,p,mod)
%LSB  Least significant bit (LSB) value in a (usually floating-point) number
%
%       [lsbmag,value]=lsb(x,p,mod)
%
%   The magnitude of bit 1 at the LSB place and value of the LSB (0 or 1).
%   In the place of p, structure of run modifiers can also be given, as
%       struct('p',p,'UnderFlow','flush to zero')
%   If the run modifier 'fb' or 'FractionBits' is given, the number is
%   treated as a fixed-point one.
%   If in the place of x a structure is given, it is treated as a
%   quantizer. For floating-point, this gives the LSB for 1.
%      mod = 'complex means for the rountine that complex lsb needs to be
%   calculated, independently of x being real or complex.

%   Examples:
%       [mag,lsbit]=lsb(1)  %is [eps = 2^(-52) = 2.2204e-016,0]
%       [mag,lsbit]=lsb(2)  %is [2*eps = 2*2^(-52) = 4.4409e-016,0]
%       [mag,lsbit]=lsb(1+2^(-23),24)  %is [eps = 2^(-23) = 1.1921e-007,1]
%       [mag,lsbit]=lsb(0)  %is [realmin*eps = 2^(-1022-52) = 4.9407e-324,0]
%       [mag,lsbit]=lsb(2^16-1,qstruct('fb',0)) %is [1,1]

%	Copyright (c) I. Kollar, 2005-2006, all rights reserved
%	Last modified: 25-Oct-2006

if nargin<3, mod=''; end
if isstruct(x)
  if length(fieldnames(x))>=13, Q=x;
  else Q=qstruct(x);
  end
  if strcmp(Q.Type,'floating-point'), [lsbmag,value]=lsb(1,Q);
  elseif strcmp(Q.Type,'fixed-point')
    [lsbmag,value]=lsb(pow2(-Q.FractionBits),Q);
  else
    error('Cannot evaluate input structure x')
  end
  if strcmp(mod,'complex'), lsbmag=lsbmag+j*lsbmag; end
  return
end

if ~isnumeric(x), error('x is not numeric'), end
if isempty(x), value=[]; lsbmag=[]; return, end
value=zeros(size(x));
lsbmag=zeros(size(x));

%p=[]; B=[]; fb=''; rdir=''; treath=''; coding=''; ovf=''; xnmin=[]; xpmax=[]; typ='';
if nargin>1
  if isnumeric(p),
    Q=qstruct('Precision',p);
  elseif isstruct(p)
    if length(fieldnames(p))>=13
      Q=p;
    else
      warning('Q is not a proper quantizer structure') %#ok<WNTAG>
      Q=qstruct(p);
    end
  elseif ischar(p)
    Q=qstruct(p);
  end
else
  Q=qstruct('double');
end

% if 0&&~isempty(Q) %eval slow!!!
%     fn=fieldnames(Q);
%     for ii=1:length(fn)
%         if strcmpi(fn{ii},'type')
%             typ=Q.Type;
%         elseif strcmpi(fn{ii},'phase')
%             ph=Q.Phase;
%         else
%             eval([fn{ii},'=getfield(Q,fn{ii});'])
%         end
%     end
% end

if ~exist('Precision','var')
  if isfield(Q,'Precision')
    Precision=Q.Precision;
  else
    Precision=[];
  end
end

if isempty(Precision), Precision=Q.Bits-~strncmp(Q.Coding,'u',1); end
for ii=1:numel(x)
  xi=x(ii);
  if any(imag(xi)), dv=1:2;
  elseif strcmp(mod,'complex'), dv=1:2;
  else dv=1;
  end
  for di=dv
    if di==1, xin=real(xi); jv=1; else xin=imag(xi); jv=j; end
    if isnan(xin)
      lsbmag(ii)=lsbmag(ii)+NaN;
    else
      if ~isempty(Q.FractionBits)&&strcmp(Q.Type,'fixed-point')
        lsbmag(ii)=lsbmag(ii)+jv*2^(-Q.FractionBits);
      else
        if ~isfinite(xin)
          lsbmag(ii)=lsbmag(ii)+jv*sign(xin)*inf;
        else
          [f,e]=log2(xin);
          if isequal(f,0)
            lsbmag(ii)=lsbmag(ii)+jv*pow2(2^(-Precision+1),Q.ExpMinimum);
          else
            lsbmag(ii)=lsbmag(ii)+jv*pow2(2^(-Precision),e);
          end
        end
      end
      if real(lsbmag(ii)/jv) ~= 0
        if rem(xin/(pow2(real(lsbmag(ii)/jv),1)),1)==0
          %value(ii)=0;
        else
          value(ii)=value(ii)+jv;
        end
      else
        %%ENTER CODE HERE FOR WHAT TO DO ON DIVISION BY ZERO
        warning('Division by zero occurred');
        value(ii) = 0;
      end
    end
  end
end

%End of lsb
