function [x,Qest]=bin2num(binstr,runmod)
%BIN2NUM Convert binary (string-) representation to Matlab-number
%
%      x=bin2num(binstr,runmod)
%
%	This is the inverse of num2bin and num2sigexp.
%	When the binary representation is not correct, warnings or error
%	   messages can be issued.
% Default quantizers may be used, except 'single': instead, 'single-roundoff' or 'single-qdsp' can be used. 
%
% Examples:
%     bin2num('+(1).0000 0000 0000 0000 0000 001 2^[+0]','single-roundoff')-1 %2^(-23) = 1.1921e-007
%     bin2num('+(0)(.)0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 (2^)0000 0000 000') %0
%     bin2num('+1.0000 0000 0000 000 e [0000 0002]') %4
%     bin2num('+(1).0000 0000 0000 000 e 0000 0000 000',struct('expc','t')) %1
%     bin2num('1111',struct('Coding','t')) %-1

%	Copyright (c) I. Kollar, 1995-2006, all rights reserved
%	Last modified: 04-Feb-2006

if (size(binstr,1)~=1)|~isstr(binstr), error('binstr is not a string'), end

%simulate IEEE double precision
qtype='o';
p=53; fb=52;
Coding='s';
overflow='i';
Emin=-1022; Emax=1023; expcoding='b'; Ebias=1-Emin; expbits=11;
ufcoding='e'; lb='h';
Name=''; op='quantize';
%
ind=[findstr('*2^',binstr),findstr('e',binstr)];
if isempty(ind), ind=findstr('2^',binstr); end
if isempty(ind)
  qtype='fixed-point'; 
  B=length(find((binstr=='0')|(binstr=='1')))+1; %|(binstr=='+')|(binstr=='-')|(binstr=='o')|(binstr=='t')));
  %B=B-sum(ismember('+-.',binstr))+1;
  first=binstr(1);
  %if strcmp(binstr(1),'.'), first=binstr(2); end
  ind=findstr('.',binstr); 
  if isempty(ind), fb=0; else fb=sum((binstr(ind+1:end)=='0')|(binstr(ind+1:end)=='1')); end
  if strcmp(first,'+')|strcmp(first,'-')
    Coding='s'; xpmax=2^(B-fb)-1; xnmin=-2^(B-fb)-1;
  elseif any(ismember(first,'10.'))
    Coding='s'; xpmax=2^(B-fb)-1; xnmin=-2^(B-fb);
  end
end
if ~exist('runmod'), runmod=''; end
nowarn=0;
if ~isempty(runmod)
  if isstr(runmod)&strncmpi(runmod,'nowarn',4) %send no warnings
    nowarn=1;
  elseif isstr(runmod)&strncmpi(runmod,'single-roundoff',6)
    if strncmpi(runmod,'single-roundoff',8)
      p=24; fb=23;
      Emin=-126; Emax=127; expcoding='b'; Ebias=1-Emin; expbits=8;
      runmod='';
    elseif strncmpi(runmod,'single-qdsp',8)&exist('qdata.m')
      if any(binstr(end)=='01.'), binstr=[binstr,'b']; end
      x=qdata(binstr,runmod);
      return
    else
      error('runmod not allowed')
    end
  elseif isnumeric(runmod)
    p=runmod; fb=p-1;
    runmod='';
  elseif isstruct(runmod)
    fn=fieldnames(runmod);
    for ii=1:length(fn)
      if strncmpi(fn{ii},'p',1), p=getfield(runmod,fn{ii});
      elseif strncmpi(fn{ii},'Emin',3)|strncmpi(fn{ii},'ExpMin',5), Emin=getfield(runmod,fn{ii});
      elseif strncmpi(fn{ii},'Ebias',4)|strncmpi(fn{ii},'ExpBias',4), Ebias=getfield(runmod,fn{ii});
      elseif strncmpi(fn{ii},'Expc',4), expcoding=getfield(runmod,fn{ii});
        if ~strncmpi(expcoding,'b',1), Ebias=0; end
      elseif strcmp(fn{ii},'lb')|strncmpi(fn{ii},'LeadingBit',3)
        lb=getfield(runmod,fn{ii}); lb=lower(lb(1));
      elseif strncmpi(fn{ii},'Name',1), Name=getfield(runmod,fn{ii});
      elseif strncmpi(fn{ii},'Bits',1), B=getfield(runmod,fn{ii});
      elseif strcmp(fn{ii},'fb')|strncmpi(fn{ii},'fractionbits',2), fb=getfield(runmod,fn{ii});
      elseif strncmpi(fn{ii},'Coding',3)
        Coding=getfield(runmod,fn{ii});
      elseif strncmpi(fn{ii},'Type',3)
      elseif strncmpi(fn{ii},'RoundingDir',9)
      elseif strncmpi(fn{ii},'TreatHalf',9)
      elseif strncmpi(fn{ii},'Dither',4)
      elseif strncmpi(fn{ii},'Minimum',3)
      elseif strncmpi(fn{ii},'Maximum',3)
      elseif strncmpi(fn{ii},'OverFlow',5)
      elseif strncmpi(fn{ii},'Operation',2)
      else error(['Unknown field ''',fn{ii},''' of runmod'])
      end
    end
  elseif isstr(runmod)&isquant(runmod)
    if any(binstr(end)=='01.'), binstr=[binstr,'b']; end
    x=qdata(binstr,runmod);
    return
  else
    error('runmod not allowed')
  end
end
%
if ~isempty(Ebias)&~isequal(Ebias,0)&(Ebias<=-Emin)
  error(sprintf('Ebias = %.0f is not larger than -Emin = %.0f',Ebias,-Emin))
end
pmax=-log2(eps)+1; x='';
if strcmp(lower(binstr),'nan'), x=NaN;
elseif strcmp(lower(binstr),'inf'), x=inf;
elseif strcmp(lower(binstr),'-inf'), x=-inf;
end
hbn=(lb=='h');
if ~isempty(x)
  Q=qstruct(Name,'double');
  return
end
%
%eliminate spaces
ind0=[find(isspace(binstr))];
if ~isempty(ind0), binstr(ind0)=''; end
%parentheses
ind0=[findstr(binstr,'('),findstr(binstr,')')];
if ~isempty(ind0)
  if any(ind0<=2), Q.LeadingBit='hidden'; end
  if any(ind0>4), Q.Operation='PQN'; end
  binstr(ind0)='';
else
  Q.LeadingBit='shown'; Q.Operation='quantize';
end
%
inde=find(lower(binstr)=='e');
if length(inde)>1, error('multiple ''e''s in binstr'), end
ind2t=findstr(binstr,'*2^');
if isempty(ind2t), ind2t=findstr(binstr,'2^'); end
if length(ind2t)>1, error('multiple ''*2^''s in binstr'), end
if ~isempty(inde)
  fstr=binstr(1:inde-1); estr=binstr(inde+1:length(binstr));
  Q.Type='floating-point';
elseif ~isempty(ind2t)
  fstr=binstr(1:ind2t-1); estr=binstr(ind2t+3:length(binstr));
  Q.Type='floating-point';
else
  fstr=binstr; estr='';
  Q.Type='fixed-point';
end
%
%Eliminate binary point
ind=find(fstr=='.');
if length(ind)>1, error('multiple ''.''s in binstr'), end
if ~isempty(ind)
  fbstrl=length(fstr(ind+1:length(fstr))); %number of fraction bits
  if fbstrl<fb
    if ~nowarn
      warning(sprintf(['Warning: Number of fraction bits is %.0f, smaller than',...
        ' fb=%.0f'],fbstrl,fb))
    end
  elseif fbstrl>fb
    if isempty(Name), qnamee='IEEEdouble'; else qnamee=Name; end
    if ~nowarn
      warning(sprintf(['Warning: Number of fraction bits is %.0f, larger than',...
        ' fb=%.0f,\n  given for quantizer ''',qnamee,''''],fbstrl,fb))
    end
  end
  fstr(ind)='';
else
  fbstrl=0;
end
if strcmp(Q.Type,'fixed-point')
  Q.fb=fbstrl; Q.B=B;
else %float
  if fbstrl==0
    Q.Precision=length(fstr);
    if strcmp(fstr(1),'+')|strcmp(fstr(1),'-'), Q.Precision=Q.Precision-1; end
  else
    Q.Precision=fbstrl+1;
  end
end
%
%Eliminate sign if given
s=1; %sign is positive by default

if strcmp(fstr(1),'+')|strcmp(fstr(1),'-'), Q.Coding='s'; 
elseif strcmp(fstr(1),'0'), Q.Coding='t';
elseif strcmp(fstr(1),'t'), Q.Coding='t';
elseif strcmp(fstr(1),'o'), Q.Coding='o';
else
  if strcmp(Q.Type,'fixed-point'), Q.Coding='s';
  else Q.Coding='s';
  end
end

if Coding=='s' %sign-magnitude representation
  if (fstr(1)=='+')|(fstr(1)=='?')|(fstr(1)=='0'), fstr(1)='0';
  elseif (fstr(1)=='-')%|(fstr(1)=='1'), 
    s=-1; fstr(1)='0';
  end
  if lb=='h' %hidden leading bit
    if length(fstr)>=4, fstr13=fstr(2:4); else fstr13=''; end
    if strcmp(fstr13,'(0)')|strcmp(fstr13,'(1)')
      fstr([2,4])=''; %eliminate parentheses
      if (ufcoding~='e')&(fstr(2)=='0')
        error('Leading bit cannot be hidden if it cannot be coded in exponent')
      end
    else
      %error('Hidden leading bit is not shown by ''(1)'' or ''(0)''')
    end
    pstr=length(fstr)-1;
  else %shown (not hidden) leading bit
    pstr=length(fstr);
  end
elseif (Coding=='o')|(Coding=='t') %one's or two's complement
  if strcmp(fstr(1),'+')|strcmp(fstr(1),'-')
    error(['String starts with ''',fstr(1),''', but Coding is ''',Coding,''''])
  elseif ((Coding=='o')&strcmp(fstr(1),'t')) | ((Coding=='t')&strcmp(fstr(1),'o'))
    error(['String starts with ''',fstr(1),''', but Coding is ''',Coding,''''])
  end
  if (fstr(1)=='1')|(fstr(1)=='t')|(fstr(1)=='o') %negative
    fstr(1)='1';
    s=-1;
    fstr=setstr('0'+'1'-fstr); %complement
    if Coding=='t'
      for fsi=length(fstr):-1:1
        bit=fstr(fsi);
        if bit=='1', fstr(fsi)='0';
        elseif bit=='0', fstr(fsi)='1'; break
        end
      end
    end
  end
  pstr=length(fstr);
elseif strcmp(Coding,'u')
  if strcmp(fstr(1),'+')|strcmp(fstr(1),'-')
    error(['String starts with ''',fstr(1),''', but Coding is ''unsigned'''])
  end
  pstr=length(fstr);
end
%pstr is p which belongs to the string
%
if strcmp(fstr(end),'*'), fstr(end)=''; end
if any((fstr(2:end)<'0')|(fstr(2:end)>'1'))|~any(fstr(1)=='to01')
  error(['Invalid bit in significand ''',fstr,''''])
end
if any(fstr(1)=='to')
  fstr(1)='1';
  fstr=setstr(['0'*ones(1,B-length(fstr)),fstr]);
end
%
ib=p-fb-1+(lb=='h'); %number of integer bits of quantizer (including hidden bit)
ibstr=pstr-fbstrl-1+(lb=='h'); %number of integer bits of string (including hb)
%
%Treat exponent
%if isempty(estr), estr='0'; end
ind1=findstr(estr,'[');
ind2=findstr(estr,']');
if ~isempty(ind1)&~isempty(ind2)
  %decimal exponent is given
  estr=estr(ind1:ind2);
  indd=findstr(estr,'d');
  if ~isempty(indd), estr(indd)=''; end
  %Ebias=0;
  e=str2num(estr);
  if ind1>1
    Q.ExpBits=ind1-1;
  else
    Q.ExpBits=ceil((length(estr)-2)*log2(10));
    if Q.ExpBits>11, Q.ExpBits=11; end
    if strcmp(estr(1),'+')|strcmp(estr(1),'-'), Q.ExpBits=Q.ExpBits+1; end
  end
end
if (isempty(ind1)|isempty(ind2))&strcmp(Q.Type,'floating-point')
  %binary with offset
  Q.ExpBits=length(estr);
  if strcmp(estr(1),'+')|strcmp(estr(1),'-'), Q.ExpCoding='t'; else Q.ExpCoding='b'; end
  se=1; eoffs=0;
  if expcoding=='s'
    if estr(1)=='+', estr(1)='';
    elseif estr(1)=='-', se=-1; estr(1)='';
    end
  elseif (expcoding=='t')|(expcoding=='o')
    if estr(1)=='1' %negative
      estr=setstr('0'+'1'-estr); se=-1;
      if expcoding=='t', eoffs=1; end
    end
  end
  if isempty(estr), e=0;
  else
    e=se*(estr-'0')*pow2(1,[length(estr)-1:-1:0]')+eoffs-Ebias;
    if ufcoding=='e' %exponent-coded underflow
      if (e==Emin-1)&(fstr(2)~='0')
        error('Coded underflow implies zero leading bit')
      elseif (e~=Emin-1)&(fstr(2)=='0')
        error('Zero leading bit is only possible with coded exponent')
      end
      if e==Emin-1, e=Emin; end
    end
    if e<Emin
      error(sprintf(['Exponent %.0f is smaller than Emin=%.0f'],e,Emin))
    elseif e>Emax
      error(sprintf(['Exponent %.0f is larger than Emax=%.0f'],e,Emax))
    end
  end
end
[dummy,eminc]=log2(realmin); eminc=eminc-1;
if isfield(Q,'ExpBits')
  Q.ExpMaximum=pow2(Q.ExpBits-1)-1;
  Q.ExpMinimum=-pow2(Q.ExpBits-1)+2;
  Q=rmfield(Q,'ExpBits');
  if isfield(Q,'ExpCoding')
    if strcmp(Q.ExpCoding,'b')
      Q.ExpBias=1-Q.ExpMinimum; 
    end
    Q=rmfield(Q,'ExpCoding');
  end
end
  %
%Go back to significand
if pstr-(lb=='s')<=pmax
  f=s*(fstr-'0')*pow2(1,[length(fstr)-1:-1:0]'-fbstrl);
else %cell
  fstrc=fstr;
  f=cell(1,2); l=pmax+1;
  f{1}=s*(fstrc(1:l)-'0')*pow2(1,[l-1:-1:0]'-fb+p-pmax-1+(lb=='h'));
  fstrc(1:l)=''; ci=1; if (qtype=='o')&(e==Emin), ufc=0; else ufc=1; end
  while ~isempty(fstrc)
    l=min(length(fstrc),pmax+1);
    f{ci+1}=s*(fstrc(1:l)-'0')*pow2(1,[l-1:-1:0]'...
      -fb+p-length(fstrc)-1+(lb=='h'));
    fstrc(1:l)=''; ci=ci+1;
  end %while
end
%
if isa(f,'cell'), f1=f{1}; else f1=f; end
if qtype=='o'
  if f1<pow2(eminc-e)
    if isa(f,'cell')
      x=f;
      for ii=1:size(f,2), x{ii}=f{ii}*2^e; end
    else %double
      x=f*2^e;
    end
    %disp('Warning! Denormalized result in bin2num')
  else %normal case
    if isa(f,'cell')
      x=f;
      for ii=1:size(f,2), x{ii}=pow2(f{ii},e); end
    else %double    
      x=pow2(f,e);
    end
  end
else %fixed-point
  %if isa(f,'cell'), x=f{1}; else x=f; end 
  x=f;
  largex=0;
  if isa(x,'double')
    if x>xpmax, largex=1; end
  elseif iscell(x)
    if (x{1}>xpmax{1})|((x{1}==xpmax{1})&(x{2}>xpmax{2}))
      largex=1;
    end
  end
  if largex==1
    error(['Number surpasses maximum'])
  end
  smallx=0;
  if isa(x,'double')
    if x<xnmin, smallx=1; end
  elseif iscell(x)
    if (x{1}<xnmin{1})|((x{1}==xnmin{1})&(x{2}<xnmin{2}))
      smallx=1;
    end
  end
  if smallx==1
    error(['Number is smaller than minimum'])
  end
end
Qest=qstruct(Name,Q);
%
%End of bin2num
