function [cf,cf1,cf2,cf3]=cfcalc(u,dist,ampl,arg2,mu)
%CFCALC Characteristic function and/or derivatives of a random variable.
%
%	[cf,cf1,cf2,cf3]=cfcalc(u,dist,ampl,arg2,mu)
%
%	The cf and derivatives are calculated via a long FFT of the pdf.
%
%	Output arguments:
%	cf = values of the characteristic function at u
%	cf1 = first derivative of the characteristic function at u
%	cf2 = second derivative of the characteristic function at u
%	cf3 = third derivative of the characteristic function at u
%
%	Input arguments:
%	u = values of the input argument (may be a vector)
%		elements of u must be integer multiples of a quantity
%	dist = distribution:
%		Chi-squared ('c' or 'chi2')
%		exponential ('e' or 'exponential')
%		gamma ('a' or 'gamma')
%		Gaussian ('g' or 'Gaussian')
%		Laplacian ('l' or 'Laplacian')
%		Rayleigh ('r' or 'Rayleigh')
%		sinusoidal ('s' or 'sinusoidal')
%		triangular ('t' or 'triangular')
%		uniform ('u' or 'uniform')
%	ampl = amplitude or standard deviation (spread control parameter)
%		reciprocal of parameter for exponential, Laplacian, gamma
%	arg2 = second parameter (if applicable, otherwise empty)
%	mu = mean value (default value: zero)
%
%       Definitions of the pdf-s and parameters:
%	Chi-squared: ampl=sigma, arg2=n,
%         f(x) = 1/sigma * ...
%               ((x/sigma)^((n-2)/2)*exp(-(x/sigma)/(2)) / (2^(n/2)*gamma(n/2))
%               x>=0, sigma>0
%	exponential: ampl=lambda,
%	  f(x) = lambda*exp(-lambda*x), x>=0, lambda>0
%	gamma: ampl=lambda, arg2=alpha,
%	  f(x) = lambda*(lambda*x)^(alpha-1)*exp(-lambda*x) / gamma(alpha)
%               x>0, alpha>0, lambda>0
%	Gaussian: ampl=sigma,
%	  f(x) = 1/(sqrt(2*pi)*sigma)*exp(-(x)^2/(2*sigma^2))
%               sigma>0
%	Laplacian: ampl=alpha,
%	  f(x) = alpha/2*exp(-alpha*abs(x)), alpha>0
%	Rayleigh: ampl=alpha
%	  f(x) = x/alpha^2*exp(-x^2/(2*alpha^2)), alpha>0, x>=0
%	sinusoidal: ampl=A,
%	  f(x) = 1/(A*pi*sqrt(1-(x/A)^2)) , abs(x)<A, A>0
%	triangular: ampl=A,
%	  f(x) = 1/A*(1-abs(x)/A), abs(x)<A, A>0
%	uniform: ampl=A,
%         f(x) = 1/(2*A), abs(x)<A, A>0
%	binary: ampl=A,
%         f(x) = 0.5*delta(x-A) + 0.5*delta(x+A)

%       Copyright (c) I. Kollar, 1994-2006
%       All rights reserved.
%       Last modified: 19-Mar-2006

if nargin<5, mu=[]; end, if isempty(mu), mu=0; end
if length(mu)>1, error('mu is not a scalar'), end
if imag(mu)~=0, error('mu is not real'), end
if nargin<4, arg2=[]; end
if nargin<3, ampl=[]; end, if isempty(ampl), ampl=1; end
if length(ampl)>1, error('ampl is not a scalar'), end
if imag(ampl)~=0, error('ampl is not real'), end
if ampl<=0, error('ampl is not positive'), end
%
if isstr(u), error('u is a string!?'), end
if any(any(imag(u)~=0)), error('u is not real'), end
if all(u==0), du0=2*pi/1; 
else
  if min(u(:))==0, du0=min(diff(sort(u(:))));
  else du0=min(diff(sort([0;u(:)])));
  end
  if du0<(max(abs(u))/1e6), error('Maximum index is larger than 1e6 in u'), end
  if abs(rem(u/du0+0.5,1)-0.5)>eps*max(u)*1e4
    error('Elements of u are not just multiples of a common divider')
  end
end
%
if strcmp(dist,'c') %chi-squared
  sigma=ampl; k=arg2;
  if isempty(k), error('Degrees of freedom undefined for chi-squared dist'), end
  xmin=0;
  if k>=2, xmax=sigma*max(k+4*sqrt(2*k),20); else xmax=sigma*10; end
  if k>=2, umax=3e3^(2/k)/sigma; else umax=6e3/sigma; end
elseif strcmp(dist,'e') %exponential
  lambda=1/ampl;
  if ~isempty(arg2), disp('Warning! arg2 is not empty in cfcalc'), end
  xmin=0; xmax=20/lambda;
  umax=3e3*lambda;
elseif strcmp(dist,'a') %gamma
  lambda=1/ampl; alpha=arg2;
  if isempty(alpha), error('Alpha is undefined for gamma distribution'), end
  xmin=0; xmax=1/lambda*max(2*alpha+4*sqrt(4*alpha),20);
  umax=5e3^(1/alpha)*lambda;
elseif strcmp(dist,'g') %Gaussian
  sigma=ampl;
  if ~isempty(arg2), disp('Warning! arg2 is not empty in cfcalc'), end
  xmin=-4*sigma; xmax=4*sigma;
  umax=5*4/sigma;
elseif strcmp(dist,'l') %Laplacian
  alpha=1/ampl;
  if ~isempty(arg2), disp('Warning! arg2 is not empty in cfcalc'), end
  xmin=-50/alpha; xmax=50/alpha;
  umax=100*alpha;
elseif strcmp(dist,'r') %Rayleigh
  alpha=ampl;
  if ~isempty(arg2), disp('Warning! arg2 is not empty in cfcalc'), end
  xmin=0; xmax=4*alpha;
  umax=1e3/alpha;
elseif strcmp(dist,'s') %sinusoidal
  A=ampl;
  if ~isempty(arg2), disp('Warning! arg2 is not empty in cfcalc'), end
  xmin=-A; xmax=A;
  umax=1e2^2/A;
elseif strcmp(dist,'t') %triangular
  A=ampl;
  if ~isempty(arg2), disp('Warning! arg2 is not empty in cfcalc'), end
  xmin=-A; xmax=A;
  umax=1e2^2/A;
elseif strcmp(dist,'u') %uniform
  A=ampl;
  if ~isempty(arg2), disp('Warning! arg2 is not empty in cfcalc'), end
  xmin=-A; xmax=A;
  umax=1e2^2/A;
else
  error(['Distribution ''',dist,''' not yet implemented in cfcalc'])
end
umax=max(umax,max(u));
%
du=min(du0,2*pi/2/(xmax-xmin));
if isfinite(du0), du=du0/pow2(nextpow2(du0/du)); end
N=pow2(nextpow2(2*umax/du+1));
if N<4096
  fprintf(['\ncfcalc, N=%.0f, dist=',dist,'\n\n'],N)
end
dx=2*pi/du/N;
if xmin==0
  x=[0:N-1]'*dx;
else
  x=[-N/2:N/2-1]'*dx;
end
%
fx=zeros(size(x));
if strcmp(dist,'c') %chi-squared
  ind=find(x>0);
  fx(ind)=1/sigma*(x(ind)/sigma).^((k-2)/2).*exp(-x(ind)/sigma/2)...
	/(pow2(k/2)*gamma(k/2));
  ind=find(x==0);
  if k==1
    fx(ind)=fx(ind+1);
  elseif k==2
    fx(ind)=ones(size(ind))*1/(pow2(k/2)*gamma(k/2))/2;
  end
elseif strcmp(dist,'e') %exponential
  ind=find(x>0);
  fx(ind)=lambda*exp(-lambda*x(ind));
  ind=find(x==0); fx(ind)=ones(size(ind))*lambda/2;
elseif strcmp(dist,'a') %gamma
  ind=find(x>0);
  fx(ind)=lambda*(lambda*x(ind)).^(alpha-1).*exp(-lambda*x(ind)) / gamma(alpha);
  if alpha==1
    ind=find(x==0); fx(ind)=ones(size(ind))*lambda/gamma(alpha)/2;
  end
elseif strcmpi(dist,'g') %Gaussian
  fx=1/sqrt(2*pi)/sigma*exp(-x.^2/2/sigma^2);
elseif strcmpi(dist,'l') %Laplacian
  fx=alpha/2*exp(-alpha*abs(x));
elseif strcmpi(dist,'r') %Rayleigh
  ind=find(x>0);
  fx(ind)=x(ind)/alpha^2.*exp(-x(ind).^2/(2*alpha^2));
elseif strcmp(dist,'s') %sinusoidal
  ind=find(abs(x)<=A-100*eps);
  fx(ind)=1/pi./sqrt(A^2-x(ind).^2);
  fxm=max(fx);
  ind=find((abs(x)>A-100*eps)&(abs(x)<=A+dx/2-100*eps));
  fx(ind)=fxm*ones(size(ind));
elseif strcmp(dist,'t') %triangular
  ind=find(abs(x)<(1-10*eps)*A);
  fx(ind)=1/A*(1-abs(x(ind))/A);
elseif strcmp(dist,'u') %uniform
  ind=find(abs(x)<(1+1e3*eps)*A);
  fx(ind)=ones(size(ind))/(2*A);
  ind=find(abs(x)>=(1-1e3*eps)*A);
  fx(ind)=0.5*fx(ind);
else
  error(['Distribution ''',dist,''' not yet implemented'])
end
%
global cfcalcplotfx
if strcmp(cfcalcplotfx,'plot')
  clg, plot(x,fx)
  axis([min(x),max(x),0,10*mean(fx)])
  xlabel(sprintf('x   (N=%.0f,  dx=%.3g)',N,dx))
  title(sprintf(['pdf for ampl=%.3g, mu=%.3g, distribution: ''',dist,''''],...
	ampl,mu))
  figure(gcf) 
  %pause
end
%
global cfcalcplotfnu
if strcmp(cfcalcplotfnu,'plot')
  if isfinite(du0), q=2*pi/du0; else q=1; end
  Nnu=round(q/dx);
  nu=[-Nnu:Nnu]'*dx;
  fnu=zeros(size(nu));
  muq=round(mu/dx)*dx;
  qiv=[floor((min(x)+muq)/q-0.5):ceil((max(x)+muq)/q+0.5)]*q;
  for qi=qiv
    qic=find(abs(qi-(x+muq))<dx/4);
    if isempty(qic), qic=find(abs(qi+q-(x+muq))<dx/4)-round(q/dx); end
    if isempty(qic), qic=find(abs(qi-q-(x+muq))<dx/4)+round(q/dx); end
    indx=find(abs(qi-(x+muq))<q/2+dx/4);
    fnu(qic-indx+Nnu+1)=fnu(qic-indx+Nnu+1)+fx(indx);
  end
  [d,indnu1]=min(abs(x+q/2)); [d,indnu2]=min(abs(x-q/2)); 
  nu=[nu;x(indnu1(1))-1e5*eps*dx;x(indnu2(1))+1e5*eps*dx]; fnu=[fnu;0;0];
  [nu,ind]=sort(nu); fnu=fnu(ind);
  plot(nu,fnu,[-q/2,-q/2,q/2,q/2],[0,1/q,1/q,0],':')
  %grid
  axis([min(nu),max(nu),0,max(2/q,2*ampl*mean(fnu))])
  xlabel(sprintf('nu   (dnu=%.3g)',dx))
  title(sprintf(['pdf of nu for ampl=%.3g, mu=%.3g, q=%.4g, dist: ''',...
	dist,''''],ampl,muq,q))
  figure(gcf) 
end %cfcalcplotnu
%
if xmin<0, cfi=conj(fft(fftshift(fx)))*dx; else cfi=conj(fft(fx))*dx; end
cf=cfi(rem(round(u/du)+N,N)+1);
%
%Checking plots
global cfcalcplotcu
if strcmp(cfcalcplotcu,'plot')
  clg, plot([0:N-1]*du,abs(cfi)), title('Calculated cf'), figure(gcf)
  pause
end
%
%clg, plot(u,abs(cf)), title('Desired cf values'), figure(gcf), pause
%
cf1=[]; cf2=[]; cf3=[];
if nargout>=2
  if xmin<0, cfi=conj(fft(fftshift(x.*fx)))*dx; else cfi=conj(fft(x.*fx))*dx;end
  cf1=(+j)*cfi(rem(round(u/du)+N,N)+1);
end
if nargout>=3
  if xmin<0, cfi=conj(fft(fftshift(x.^2.*fx)))*dx;
  else cfi=conj(fft(x.^2.*fx))*dx;
  end
  cf2=(+j)^2*cfi(rem(round(u/du)+N,N)+1);
end
if nargout>=4
  if xmin<0, cfi=conj(fft(fftshift(x.^3.*fx)))*dx;
  else cfi=conj(fft(x.^3.*fx))*dx;
  end
  cf3=(+j)^3*cfi(rem(round(u/du)+N,N)+1);
end
%
%Combine with mu if it differs from zero
if mu~=0
  [cf,cf1,cf2,cf3]=cfmu(u,mu,cf,cf1,cf2,cf3);
end %mu~=0
%
%End of cfcalc
