function [x,dexp]=expand(y,Delta,mode,muorA)
%EXPAND  EXPANDER for floating point quantization
%
%        [x,dexp]=expand(y,Delta,mode,muorA)
%
%        Delta: parameter of the compressor (see book "Quantization Noise")
%           If string, 'd*' means IEEE double-precision floating-point: Delta=2^(-1022), 
%           's*' means IEEE single precision fp: Delta=2^(-126)
%           Default: Delta = 1
%        mode: '' or 'p*' or 'f*' for piecewise linear floating-point compressor,
%           'u' (upper) for inverse of upper approximate compressor (x=y for -2*Delta <= y <= 2*Delta)
%           'l' (lower) for inverse of lower approximate compressor (x=y for -1.5*Delta <= y <= 1.5*Delta)
%           's' (sinh) or 'a' (asinh) for sinh approximation
%           'm' (mu-law) or 'A' (A-law)
%                For these cases V=128*Delta, and a last parameter (mu or A) may also be given. 
%                Defaults: mu = 255, A = 87.6
%                The input can be rescaled to match the other compressors 
%                by y/((1+log_2(V))/V) = 16*y 
%           Default: 'piecewise'
%        dexp: derivative of the exponent at the given points
%           (not yet implemented for all)
%
%        Example (delete the unnecessary):
%          y=[0:0.01:8]; cla, hold off
%          plot(y,expand(y,1,'asinh'),'k'), hold on
%          plot(y,expand(y,1,'u'),'r')
%          plot(y,expand(y,1,'lower'),'g')
%          plot(y,expand(y*16,1,'mu-law'),'c')
%          plot(y,expand(y*16,1,'A-law'),'m')
%          plot(y,expand(y,1,'p'),'b')
%          hold off, zoom on, shg
%
%        See also COMPRESS

%	Copyright (c) I. Kollar, 2005, all rights reserved
%	Last modified: 19-Dec-2005

if nargin<2, Delta=[]; end, if isempty(Delta), Delta=1; end
if isstr(Delta)
  if strncmp(Delta,'double',1), Delta=2^(-1022);
  elseif strncmp(Delta,'single',1), Delta=2^(-126);
  else error(['Delta is not allowed as ''',Delta,''''])
  end
end
if ~isnumeric(Delta)|(length(Delta)~=1), error('Delta is not a scalar'), end
if (Delta<=0)|~isfinite(Delta), error('Delta is not positive'), end
%
if nargin<3, mode=''; end, if isempty(mode), mode='piecewise'; end
if any(isnan(mode)), error('mode is not a string'), end
%
if nargin<4, muorA=[]; end
%
if strncmp(mode,'piecewise',1)|strncmp(mode,'floating',1)
  %piecewise linear
  k=floor(abs(y)/Delta)-1;
  x=pow2(y-sign(y).*k*Delta,k);
  ind=find(abs(y)<2*Delta);
  x(ind)=y(ind);
  %
  if nargout>1
    dexp=Delta*2.^(floor(abs(y/Delta))-1);
    %jumps:
    indm=find(rem(y,Delta)==0); 
    if ~isempty(indm)
      dexp(indm)=Delta*0.75*2.^(floor(abs(y(indm)/Delta))-1);
    end
    %center:
    dexp(ind)=ones(size(ind));
  end
elseif strncmp(mode,'asinh',1)|strncmp(mode,'sinh',1)
  %sinh expander
  %Compressor: y=Delta*asinh(x/Delta)/log(2);
  x=Delta*sinh(y/Delta*log(2));
  %
  if nargout>1
    dexp=log(2)*cosh(y/Delta*log(2));
  end
elseif strncmp(mode,'upper',1)
  %inverse of approximate compressor for floating point quantization
  x=Delta*sign(y).*2.^(abs(y)/Delta-1);
  ind=find(abs(y)<2*Delta);
  x(ind)=y(ind);
  %Compressor:
  %y=Delta*sign(x).*(log(abs(x+realmin)/Delta)/log(2)+1);
  %ind=find(abs(x)<2*Delta);
  %y(ind)=x(ind);
  if nargout>1
    dexp=log(2)*2.^(abs(y)/Delta-1);
    ind=find(abs(y)<2*Delta);
    dexp(ind)=ones(size(ind));
  end
elseif strncmp(mode,'lower',1)
  %inverse of lower approximating compressor for floating point quantization
  x=(Delta*sign(y).*3/4).*(2.^(abs(y)/Delta-0.5));
  ind=find(abs(y)<=1.5*Delta);
  x(ind)=y(ind);
  %Compressor:
  %y=Delta*sign(x).*(log(abs(4/3*x+realmin)/Delta)/log(2)+0.5);
  %ind=find(abs(x)<=1.5*Delta);
  %y(ind)=x(ind);
  %
  if nargout>1
    dexp=3/4*log(2)*2.^(abs(y)/Delta-0.5);
    ind=find(abs(y)<1.5*Delta);
    dexp(ind)=ones(size(ind));
  end
elseif strncmp(mode,'mu-law',1)
  %mu-law 
  mu=255; V=128*Delta;
  if any(abs(y)>V), warning('For an element of y, y < -V  or  V < y'), end
  if ~isempty(muorA)
    if isnumeric(muorA)&isfinite(muorA)&(length(muorA)==1)&(muorA>0), mu=muorA; 
    else error(sprintf('mu = %.0g is not allowed',muorA))
    end
  end
  x=((exp((abs(y)/V)*log(1+mu))-1)/mu*V).*sign(y);
  %Compressor:
  %y=V*log(1+mu*abs(x)/V)./log(1+mu).*sign(x);
  %
  if nargout>1
    error('Derivative is not yet ready')
  end
elseif strncmp(mode,'A-law',1)
  %A-law
  A=87.6; V=128*Delta;
  if any(abs(y)>V), warning('For an element of y, y < -V  or  V < y'), end
  if ~isempty(muorA)
    if isnumeric(muorA)&isfinite(muorA)&(length(muorA)==1)&(muorA>1), A=muorA; 
    else error(sprintf('A = %.0g is not allowed',muorA))
    end
  end
  x=(exp(abs(y)/V*(1+log(A))-1)/A*V).*sign(y);
  %Compressor:
  %y=V*(1+log(A*(abs(x)+realmin)/V))./(1+log(A)).*sign(x);
  %ind=abs(x)<V/A; if ~isempty(ind), y(ind)=A/(1+log(A))*x(ind); end
  %
  if nargout>1
    error('Derivative is not yet ready')
  end
else
  error(['mode is not allowed as ''',mode,''''])
end