function [cf,cf1,cf2,cf3]=cfgamma(u,lambda,alpha,mu)
%CFGAMMA Characteristic function of a gamma-distributed random variable.
%
%	[cf,cf1,cf2,cf3]=cfgamma(u,lambda,alpha,mu)
%
%	Definitions:
%	f(x) = lambda*(lambda*x)^(alpha-1)*exp(-lambda*x) / gamma(alpha)
%		x>0, alpha>0, lambda>0
%	F(u) = 1 / (1-j*u/lambda)^alpha
%
%	Output arguments:
%       cf = values of the characteristic function at u
%       cf1 = first derivative of the characteristic function at u
%       cf2 = second derivative of the characteristic function at u
%       cf3 = third derivative of the characteristic function at u
%
%	Input arguments:
%	u = values of the input argument (may be a vector or an array)
%	lambda = "amplitude" parameter of the distribution
%	alpha = second parameter of the distribution
%	mu = additional mean value (default value: zero)
%
%	See also: CFCHI2, CFEXPON, CFCALC

%       Copyright (c) I. Kollar, 1994
%       All rights reserved.
%       Last modified: 01-Dec-1994

if nargin<4, mu=[]; end, if isempty(mu), mu=0; end
if length(mu)>1, error('mu is not a scalar'), end
if imag(mu)~=0, error('mu is not real'), end
if nargin<3, error('alpha is not given'), end
if length(alpha)>1, error('alpha is not a scalar'), end
if imag(alpha)~=0, error('alpha is not real'), end
if alpha<=0, error('alpha is not positive'), end
if nargin<2, lambda=[]; end, if isempty(lambda), lambda=1; end
if length(lambda)>1, error('lambda is not a scalar'), end
if imag(lambda)~=0, error('lambda is not real'), end
if lambda<=0, error('lambda is not positive'), end
if any(any(imag(u)~=0)), error('u is not real'), end
%
if isfinite(lambda), cf=1./(1-j/lambda*u).^alpha;
elseif isnan(lambda), cf=NaN*ones(size(u));
else cf=ones(size(u));
end
cf1=[]; cf2=[]; cf3=[];
if nargout>=2
  term=1./(1-j/lambda*u);
  cf1=(alpha)*j/lambda*cf.*term;
end
if nargout>=3
  cf2=(alpha+1)*j/lambda*cf1.*term;
end
if nargout>=4
  cf3=(alpha+2)*j/lambda*cf2.*term;
end
%
%Combine with mu if it differs from zero
if mu~=0
  [cf,cf1,cf2,cf3]=cfmu(u,mu,cf,cf1,cf2,cf3);
end %mu~=0
%
%End of cfgamma
