function [xprime,ovfind]=qfloat(x,varargin)
%QFLOAT Floating-point quantizer, IEEE-like conventions
%
%   xprime=qfloat(x,runmod) executes floating-point rounding (quantization) 
%      on real or complex, scalar or array x.
%   xprime=qfloat(x,p) changes IEEE double precision to precision p.
%      Other parameters are set according the the IEEE standard.
%   xprime=qfloat(x,'single') changes IEEE double precision to single precision.
%      qfloat(x,runmod,'list') also lists the settings of the quantizer
%      implemented.
%   Possibilities for parameters in the structure runmod:
%     runmod.p: precision: number of bits of the significand, sign not
%         included, but hidden bit is included
%	    runmod.lb: leading bit representation, 'h' or 'hidden' for hidden, 
%         's' or 'shown' for shown
%     runmod.rdir: direction of quantization: 'r', 'f', 'c' 'x', 'i', 't'
%         for round, floor, ceil, fix, toinf, trunc
%     runmod.treath: way of treating *.5 values for round: 'f', 'c', 'x', 'e', 'o', 'r'
%         for floor, ceil, fix, toinf, toeven, toodd, rand
%     runmod.coding = coding of mantissa, 's', 'o', 't'
%         for sign-magnitude, one's complement, two's complement 
%     runmod.ufl: underflow, 'g' for gradual, 'f' for flush
%     runmod.Emin: minimum value of exponent
%     runmod.Emax: maximum value of exponent
%   Default values:
%     IEEE double precision: p=53, lb='hidden', rdir='round',
%         treath='toeven', coding='s', ufl='gradual', Emin=-1022, Emax=1023
%     IEEE single precision: p=24, lb='hidden', rdir='round',
%         treath='toeven', coding='s', ufl='gradual', Emin=-126, Emax=127
%     Limits: p<=53, Emin>=-1022, Emax<=1023
%   Examples:
%     qfloat(1+2^(-24),'s')-1  %returns 0 
%     qfloat(1+2^(-23),'s')-1  %returns 2^(-23)=1.1921e-007
%     qfloat(realmin/2,qstruct('ufl','f'),'list')  %returns 0
%     qfloat(realmin/2,qstruct('ufl','g'))  %returns  realmin/2=1.1125e-308
%   Remark: type 'qfloat remark'

%	Copyright (c) I. Kollar, 1995-2005, all rights reserved
%	Last modified: 13-Nov-2005

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Old-form call:
%function [xprime,ovfind]=qfloat(x,p,lb,rdir,treath,coding,ufl,Emin,Emax)
%QFLOAT Floating-point quantizer, IEEE-like conventions, with some parameters.
%	FractBit=p-1 is assumed, and OverFlow='inf' (use inf's)
%
%	[xprime,ovfind]=qfloat(x,p,lb,rdir,treath,coding,ufl,Emin,Emax)
%
%	Input arguments:
%	x = input value(s), scalar, vector or array, real or complex
%	p = precision: number of bits in the storage of the significand, sign
%		included
% Optional input arguments:
%	lb = leading bit representation, 'h' for hidden, 's' for shown
%	rdir = direction of quantization: 'r', 'f', 'c' 'x', 'i', 't'
%              for round, floor, ceil, fix, toinf, trunc
%	treath = way of treating *.5 values for round: 'f', 'c', 'x', 'e', 'o', 'r'
%              for floor, ceil, fix, toinf, toeven, toodd, rand
%	coding = coding of mantissa, 's', 'o', 't'
%              for sign-magnitude, one's complement, two's complement 
%	ufl = UnderFlow, 'g' for gradual, 'f' for flush
%	Emin = minimum value of exponent
%	Emax = maximum value of exponent
%
%	Output arguments:
%	xprime = quantized output value(s)
%	ovfind = indices of elements of x which cause overflow, indices
%		of negative overflow are multiplied by (-1), indices of
%		imaginary parts furthermore by sqrt(-1)
%
%	Default values: lb='h', rdir='r', treath='e', coding='s', ufl='g',
%                       Computer limits for Emin, Emax: [-1022,1023] for IEEE.

if (nargin==1)&isstr(x)&strncmpi(x,'remark',3), type qf_remark.m, return, end %remark
if nargin>1
    if isstr(varargin{1})&strcmp(varargin{1},'list'), Q=[];
    else
      if isstruct(varargin{1}), Q=varargin{1};
      else Q=qstruct(varargin{1});
      end
    end
  if isstruct(Q)&isfield(Q,'Type')&strncmp(Q.Type,'fixed-point',3)
    [xprime,ovfind]=qfix(x,Q);
    return
  end
else
  Q=qstruct('double');
end

ovfind=[];
makeqlist=''; %list quantizer parameters
if nargin>=2
  if isstr(varargin{end})
    if strcmp(varargin{end},'list')
      makeqlist='list'; varargin(end)=[];
    end
  end
end
%
if ~isempty(Q) 
  fn=fieldnames(Q);
  for ii=1:length(fn)
    if strcmp(fn{ii},'Type'), typ=Q.Type;
      %else eval([fn{ii},'=getfield(Q,fn{ii});'])
    elseif strcmp(fn{ii},'Name'), name=Q.Name;
    elseif strcmp(fn{ii},'Precision'), p=Q.Precision;
    elseif strcmp(fn{ii},'Bits'), B=Q.Bits;
    elseif strcmp(fn{ii},'FractionBits'), fb=Q.FractionBits;
    elseif strcmp(fn{ii},'RoundingDir'), rdir=Q.RoundingDir;
    elseif strcmp(fn{ii},'TreatHalf'), treath=Q.TreatHalf;
    elseif strcmp(fn{ii},'Coding'), coding=Q.Coding;
    elseif strcmp(fn{ii},'OverFlow'), ovf=Q.OverFlow;
    elseif strcmp(fn{ii},'Minimum'), xnmin=Q.Minimum;
    elseif strcmp(fn{ii},'Maximum'), xpmax=Q.Maximum;
    elseif strcmp(fn{ii},'ExpMinimum'), Emin=Q.ExpMinimum;
    elseif strcmp(fn{ii},'ExpMaximum'), Emax=Q.ExpMaximum;
    elseif strcmp(fn{ii},'ExpBias'), Ebias=Q.ExpBias;
    elseif strcmp(fn{ii},'Operation'), op=Q.Operation;
    elseif strcmp(fn{ii},'LeadingBit'), lb=Q.LeadingBit;
    elseif strcmp(fn{ii},'UnderFlow'), ufl=Q.UnderFlow;
      %
    elseif strcmpi(fn{ii},'Dither'), dith=Q.Dither;
    elseif strcmpi(fn{ii},'ToD'), tod=Q.ToD;
    elseif strcmpi(fn{ii},'Number_of_DitherBits'), L=Q.Number_of_DitherBits;
    elseif strcmpi(fn{ii},'Amplitude_of_Sinusoid_Dither'), A=Q.Amplitude_of_Sinusoid_Dither;
    elseif strcmpi(fn{ii},'Ratio_of_Frequencies')
      RoFn=Q.Ratio_of_Frequencies_numerator;
      RoFd=Q.Ratio_of_Frequencies_denumerator;
    elseif strcmpi(fn{ii},'Phase'), ph=Q.Phase;
    elseif strcmpi(fn{ii},'Variance_of_Gaussian_Dither'), gvar=Q.Variance_of_Gaussian_Dither;
    elseif strcmpi(fn{ii},'DFunction'), dfun=Q.Dfunction;
    elseif strcmpi(fn{ii},'Sequenceperm'), seqp=Q.Sequenceperm;
    elseif strcmpi(fn{ii},'Other'), other=Q.Other;
    else error(['Unknown field ''',fn{ii},''''])
    end
  end
else %numeric p or old call
  if ~isempty(varargin), p = varargin{1}; end
  if length(varargin)>=2, lb=varargin{2}; end
  if length(varargin)>=3, rdir=varargin{3}; end
  if length(varargin)>=4, treath=varargin{4}; end
  if length(varargin)>=5, coding=varargin{5}; end
  if length(varargin)>=6, ufl=varargin{6}; end
  if length(varargin)>=7, Emin=varargin{7}; end
  if length(varargin)>=8, Emax=varargin{8}; end
end
%
pmax=-log2(eps)+1; %53 for IEEE double precision
if ~exist('p'), p=[]; end, if isempty(p), p=53; end
if rem(p,1)~=0, error('p is not integer'), end
fb=p-1;
%
if ~exist('lb'), lb=''; end, if isnan(lb), lb=''; end
if isempty(lb), lb='h'; end %hidden leading bit
if strncmp(lb,'h',1), hbn=1; 
elseif strncmp(lb,'s',1), hbn=0;
else error(['lb=''',lb,''' is not allowed'])
end
if p>pmax, error(sprintf('bit number p is larger than %.0f',pmax)), end
if p<2, error('bit number of significand is not positive'), end
%
if ~exist('rdir'), rdir=''; end, if isnan(rdir), rdir=''; end
if isempty(rdir), rdir='r'; end %round
if ~exist('treath'), treath=''; end, if isnan(treath), treath=''; end
if isempty(treath), treath='e'; end %round to even
if ~exist('coding'), coding=''; end, if isempty(coding), coding='s'; end %sign-magnitude
if ~exist('ufl'), ufl=''; end, if isnan(ufl), ufl=''; end
if isempty(ufl), ufl='g'; end %gradual underflow
%
Eminlim=round(log2(realmin)); Emaxlim=round(log2(realmax))-1;
%For IEEE double precision, Emin=-1022, Emax=1023
if ~exist('Emin'), Emin=[]; end, if isnan(Emin), Emin=[]; end
if isempty(Emin), Emin=Eminlim; end
if ~exist('Emax'), Emax=[]; end, if isnan(Emax), Emax=[]; end
if isempty(Emax), Emax=Emaxlim; end
if Emin<Eminlim
  error(sprintf('Emin=%.3g is too small for this computer, limit: %.0f',...
	Emin,Eminlim))
end
if Emax>Emaxlim
  error(sprintf('Emax=%.3g is too large for this computer, limit: %.0f',...
	Emax,Emaxlim))
end
if Emax<Emin, error('Emax is smaller than Emin'), end
%
%if length(varargin)==0
%   %Do not modify IEEE double floating-point numbers
%  xprime=x; ovf=[]; return
%end
%
xsize=size(x);
if all(imag(x(:))==0) %real x
  xcompl=0;
else %complex x: split real and imaginary parts for simpler work
  x=[real(x(:)),imag(x(:))];
  xcompl=1;
end
%
%Calculate significand and exponent
[f,e]=log2(x);
f=f*2; e=e-1; %normalize for IEEE definition: significand is 1 <= s < 2
if strncmp(coding,'t',1) %For two's complement, one more negative number can be represented 
  indh=find(f==-1); %significands: -1 -> -2
  if ~isempty(indh), f(indh)=2*f(indh); e(indh)=e(indh)-1; end
end
%
ver=version; v=str2num(ver(1:3));
if v<5.3
  error('This function does not work under Matlab version 5.3')
end
%
%Bypass bug for denormalized numbers (in Matlab5?)
ind=find((abs(x)<realmin)&(x~=0));
if ~isempty(ind)
  [fden,eden]=log2(x(ind)*2^52); fden=fden*2; eden=eden-1-52;
  ind2=find(eden~=Emin);
  fden(ind2)=pow2(fden(ind2),eden(ind2)-Emin);
  eden(ind2)=Emin*ones(size(ind2));
  f(ind)=fden; e(ind)=eden;
end
indsmall=find(abs(f)<1); %underflow
if ~isempty(indsmall)&strncmp(ufl,'f',1) %if flush to zero
  f(indsmall)=pow2(f(indsmall),-(p-2+hbn)); %will need restoration at the end
end
%
f=pow2(f,p-2+hbn); %prepare for rounding by proper scaling
if (p-1+hbn<pmax)
  %Only if p<53 for ieee
  %Main quantization operation
  if strncmp(rdir,'r',1) %rounding
    if strncmp(treath,'e',1)|strncmp(treath,'toeven',1) %shortcut for convergent (round to even) rounding
      %adding an '1' properly will round the lower bits
      fprime=(f+sign(f)*pow2(pmax-1))-sign(f)*pow2(pmax-1);
    else %not convergent ieee
      fprime=round(f);
      hind=find(abs(rem(f,1))-0.5==0);
      if ~isempty(hind) %half-values present
        if strncmp(treath,'x',1)
          fprime(hind)=fix(f(hind));
        elseif strncmp(treath,'c',1)
          fprime(hind)=ceil(f(hind));
        elseif strncmp(treath,'f',1)
          fprime(hind)=floor(f(hind));
        elseif strncmp(treath,'i',1), fprime(hind)=-fix(-f(hind)); %toinf
        elseif strncmp(treath,'e',1)|strncmp(treath,'o',1) %toeven,toodd
          uind=find(abs(rem(f(hind),2))>1); uind=hind(uind);
          lind=find(abs(rem(f(hind),2))<1); lind=hind(lind);
          if strncmp(treath,'e',1)
            fprime(uind)=fix(f(uind)+sign(f(uind))); 
            fprime(lind)=fix(f(lind)); 
          else
            fprime(lind)=fix(f(lind)+sign(f(lind))); 
            fprime(uind)=fix(f(uind)); 
          end
        elseif strncmp(treath,'r',1)
          fprime(hind)=round(f(hind)+rand(size(hind)));
        else error(['treath=''',treath,''' is not allowed'])
        end
      end %half-values
    end %not convergent ieee
  elseif strncmp(rdir,'i',1) %toinf
    fprime=ceil(f); nind=find(x<0); fprime(nind)=-ceil(-f(nind));
  elseif strncmp(rdir,'fix',3)|strncmp(rdir,'x',1)|(strncmp(rdir,'t',1)&strncmp(coding,'s',1))
    fprime=fix(f); %fix or trunc
  elseif strncmp(rdir,'c',1), fprime=ceil(f);
  elseif strncmp(rdir,'fl',2)|(strncmp(rdir,'t',1)&any(coding(1)=='ot')), fprime=floor(f);
  else error(['rdir ''',rdir,''' is not allowed'])
  end
else %p=pmax: no quantization. Output should be identical to input.
  fprime=f;
end
%
%Restore underflow numbers for flush to zero
if ~isempty(indsmall)&strncmp(ufl,'f',1) %flush to zero
  fprime(indsmall)=pow2(fprime(indsmall),p-2+hbn);
end
%
ind=find((fprime~=0)&(e-(p-2+hbn)<Emin)); %find denormalized numbers
if ~isempty(ind), fprime(ind)=fprime(ind)*2^52; end %Bypass bug for denorm
xprime=pow2(fprime,e-(p-2+hbn));
if ~isempty(ind), xprime(ind)=xprime(ind)/2^52; end
%
%Set too large numbers to inf
xmax=pow2(pow2(p+hbn)-1,Emax-fb);
if strncmp(coding,'t',1), xmin=-pow2(pow2(p+hbn),Emax-fb);
else xmin=-xmax;
end
indlarge=find((xprime>xmax)|(xprime<xmin)); %numbers to be clipped
if ~isempty(indlarge)
  xprime(indlarge)=inf*sign(f(indlarge));
end
%
%Set too small numbers to zero
if strcmp(ufl,'f') %flush to zero
  ind=find(abs(xprime)<=pow2(Emin-1));
else %gradual
  ind=find(abs(xprime)*2<=pow2(Emin-p+1));
end
if ~isempty(ind), xprime(ind)=zeros(size(ind)); end
%
%Mark overflows
%if nargout>1
%  ovfind=[find(xprime==inf),-find(xprime==-inf)]; ovfind=ovfind(:);
%  if ~isempty(ovfind), [dummy,ind]=sort(abs(ovfind)); ovfind=ovfind(ind); end
%end
%
if xcompl %complex x
  xprime=xprime(:,1)+j*xprime(:,2);
  xprime=reshape(xprime,xsize);
  indimag=find(ovfind>prod(size(x)));
  if ~isempty(indimag)
    ovfind(indimag)=j*(ovfind(indimag)-prod(size(x)));
  end
end
%
%make list
if strcmp(makeqlist,'list')
  qstructget(varargin{:})
  %Show quantized number
  if (length(xprime)==1)&~xcompl
    fprintf('  '), num2bin(xprime,struct('p',p,'Emin',Emin),'d');
  end
end
%
%End of qfloat
